#!/bin/bash


# version 0.2a. 20230205. copyleft.

# from 0.2:
# 1. bug fixed [sed -n '/Date/,/files/p'] to [sed -n '/Date/,/\lfile/p'].
# 2. found the $IFS bug but not yet resolved.

# purposes:
# 1. dig out all the "$1" files(only show results of file path/name) under current working dir/subdir and
#     including which packed within archive files even within archive of archive of archive of...
#
# 2. the same as 1., and also search contents of the matched files by pattern "$2", which is also in grep style pattern and
#     including options, e.g., $2=="'.*void +main\(.*\)' -i -n -B5 -A5". (note, the options in trailing is must).
#
# 3. the same as 2., and copy the matched files to the assigned absolute-directory by "$3".

# required:
# 1. apt install p7zip-full p7zip-rar
# 2. in the /tmp/ dir for extracting files for temporary use, auto deleted after done.

# known limitation:
# 1. the search pattern is in grep format.
# 2. search twice if the soft-link and its real-path are both in the searching-paths.
# 3. multi-volume archive will cause 7z to exit. so rename existing ones are needed before running this script into completely.
# 4. it is used by "7z -px" to skip password protected archives.
# 5. it is used by "7z -aou" to auto rename repetitive extracted files.
# 6. token 'null' is used, so it is not for pattern search.
# 7. file descriptor 3 is used for identification purpose, so do not use it before running this script.
# 8. except parameters $1, $2, $3 are for user, the other params are only for internal use:
#     "$4" passing a specific archive file for extracting and searching.
#     "$5" script path.
#     "$6" script name.
# 9. $2, the content pattern, must have the options in trailing. that is like as "'the_pattern' -opt1 -opt2".

# example1: [this_script ".*\.pdf$"]
#     find out all the "pdf" files where locate including sub-dirs and within archive files.
#
# example2: detach the task from shell for free run.
# [sudo nohup this_sh ".*\.pdf$\|.*\.doc$\|.*\.chm$\|.*\.djvu$\|.*\.pptx\?$\|.*\.pps$\|.*\.xlsx\?$\|.*\.mht$" &> ~/Output.txt & disown]
#
# example3: [this_script ".*\.c$" "'.*printf.*' -n -B3 -A5"]
#     find out all the "*.c" files which have pattern '.*printf.*' in it. and
#     also dump out these line-number and the before 3 lines and the after 5 lines.
#
# example4: [this_script ".*\.c$" "'.*printf.*' -n -B3 -A5" /home/user/Desktop/target]
#     the same as example3 and copy matched files to folder /home/user/Desktop/target/.




lsof -a -p $$ -d 3 2>/dev/null | grep -i -q "\ 3w\ \|\ 3u\ "    # use file descriptor 3 for recognizing root

if [ $? -eq 0 ]; then                                   # if not the root task

    if [ $# -eq 6 ]; then                               # if intends for fork-task
        inipath=$( pwd )
        arcname="$( basename "$4" ).XXXXXXXX"
        arcpath=$( mktemp -t -d "$arcname" )
        SCRIPT_DIR="$5"
        SCRIPT_NAME="$6"
        7z -px -aou -o"$arcpath" x "$4" 2>/dev/null 1>/dev/null
        cd "$arcpath"
    else                                                # something wrong
        exit 1
    fi

else                                                    # the root task, generates fd3

    # trap prevents from aborted by user and $IFS not yet recovered
    IFS_OLD="$IFS"
    function for_trap_exit() {
        echo -e '\n\nuser abort\n';
        [[ "$IFS_OLD" != "$IFS" ]] && IFS="$IFS_OLD" && echo clean up;
        # be careful not to clean-up /tmp/ here or get into catastrophe since I met.
        exec 3>&-;
        exit 1;
    }
    trap for_trap_exit SIGINT SIGKILL

    # wrong arguments
    if [ $# -lt 1 ] || [ $# -gt 3 ]; then
        echo; echo "usage: ${0} \"filename pattern for search\" [\"context pattern in file\" [\"path to extract if match\"] ]";
        exit 1
    fi

    # no 7z bin file
    7z | grep -i "copyright"
    if [ $? -ne 0 ]; then
        echo; echo "utility 7z is needed. please try \"apt install p7zip-full p7zip-rar\" first.";
	exit 1
    fi

    # set global vars
    SCRIPT_DIR=$( cd -- "$( dirname -- "${BASH_SOURCE[0]}" )" &> /dev/null && pwd )
    SCRIPT_NAME=$( basename -- "${BASH_SOURCE[0]}" )
    INI_DIR=$( pwd )
    inipath=$( pwd )
    arcpath=$( pwd )

    # rearrange arguments
    z1="$1"
    if [ $# -eq 1 ]; then
        z2='null'
        z3='null'
    elif [ $# -eq 2 ]; then
        if [ "$2" = 'null' ]; then
            echo '"null" not applicable.'
            exit 1
        fi
        z2="$2"
        z3='null'
    else
        if [ "$2" = 'null' ] || [ "$3" = 'null' ]; then
            echo '"null" not applicable.'
            exit 1
        fi
        z2="$2"
        z3="$3"
        echo "$z3" | grep -q "^/.*"
        if [ $? -eq 0 ] && [ -d "$z3" ]; then

            # try to padding "/"
            echo "$z3" | grep -q ".*/$"
            [[ $? -ne 0 ]] && z3="${z3}/"

        else
            echo; echo "the absolute path is needed."; echo;
            exit 1
        fi
    fi

    set --
    set "$z1" "$z2" "$z3"
    exec 3>&1
    echo; echo $( date ); echo;

fi


if [ "$2" != $'null' ]; then                            # needs for context search
    indivCS=$( find -L ~+ -type f -regextype grep -iregex "$1" )

    IFS_OLD="$IFS"
    IFS=$'\n'
    for ctx in $indivCS; do
IFS=$IFS_OLD  ########################## temporarily added ################################################
        grep $2 -- "$ctx"     ############### the bug is here ###############################################################
        #exec grep $2 -- "$ctx"
        #grep $2 "$ctx"                                  # note the $2 not quoted for options injection

        if [ $? -eq 0 ]; then
            echo "$ctx    [[[Found Here]]]"; echo -e "----------------\n";
            if [ "$3" != $'null' ]; then
                i=0;
                j=$( basename -- "$ctx" )
                k=$j
                while [ -e "${3}${k}" ]; do
                    (( i=$i+1 ));
		    k="$j($i)"
                done
		cp "$ctx" "${3}${k}"
            fi
        fi
IFS=$'\n'  ################################## temporarily added ########################################
    done
    IFS="$IFS_OLD"
else                                                    # only for file name search
    find -L ~+ -type f -regextype grep -iregex "$1" | sed 's/\(.*\)/\"\1\"/' | xargs -n1 # list all the existing pdfs
fi


t=$( find -L ~+ -type f -regextype sed -iregex '.*\.\(zip\|rar\|ace\|arj\|t\?gz\|tar\|z\|7z\|xz\|bz2\|lzh\|ex[e_]\{1\}\|iso\)' | sed 's/\(.*\)/\"\1\"/' | xargs -n1 )

IFS_OLD="$IFS"
IFS=$'\n'
for tt in $t; do

    echo -e "----->>>>>  $tt\n"

    ttt=$( 7z -px l "$tt" | sed -n '/Date/,/\lfile/p' )
    v=$( echo "$ttt" | grep -i "\.zip$\|\.rar$\|\.ace$\|\.arj$\|\.t\?gz$\|\.tar$\|\.z$\|\.7z$\|\.xz$\|\.bz2$\|\.lzh$\|\.ex[e_]\{1\}$\|\.iso$" )
    if [ $? -eq 0 ]; then                               # need to extract further for advanced search since another package inside
        IFS="$IFS_OLD"                                  # branch out then need recovery
        "$SCRIPT_DIR"/"$SCRIPT_NAME" "$1" "$2" "$3" "$tt" "$SCRIPT_DIR" "$SCRIPT_NAME"
        IFS=$'\n'                                       # branch in then need recovery
    elif [ "$2" != $'null' ]; then                      # need to extract further if at least one match for context search
        echo "$ttt" | grep -m1 -q -i "$1"
        if [ $? -eq 0 ]; then
            IFS="$IFS_OLD"
            "$SCRIPT_DIR"/"$SCRIPT_NAME" "$1" "$2" "$3" "$tt" "$SCRIPT_DIR" "$SCRIPT_NAME"
            IFS=$'\n'
	fi
    else
        echo "$ttt" | grep -i "$1"
    fi

done
IFS="$IFS_OLD"

if [ "$inipath" != "$arcpath" ]; then                   # if not the root
    cd "$inipath"
    rm -rf "$arcpath"
else
    exec 3>&-                                           # done the search and release fd3
    echo; echo $( date ); echo;
fi

# end of sh
